//Common Libraries
#include "AlphaDriver.h"
#include "AlphaInterface.h"
#include "ModuleTypes.h" 
#include "ClockAndTemp.h"
#include <stdio.h> 
#include <string.h>
#include <time.h>
//Enables or disables Initial clearing of customer data
#define ClearMaxTemperature 0//1 for clear, 0 for don't clear data
#define baud 19200// Define the baud rate that you want to use

#ifdef __GNUC__	
//Linux Libraries
#include "LinuxFunctions.h"
#include <termios.h>
#include <unistd.h> 
#include <fcntl.h> 
#include <sys/signal.h> 
#include <sys/types.h> 
#include <sys/select.h> 
#define _kbhit KbhitForLinux
//#define sprintf snprintf
#define Sleep MilliSecondSleep
#define PORTNAME "/dev/ttyUSB0"
#else	

//Windows Libraries	
#include <conio.h>
#include <stdint.h>
#include <windows.h>
#define PORTNAME "\\\\.\\COM54"
#endif

//Main Code
int main(int argc, char ** argv)
{	
	int8_t currentTemperature = 0;//Current temperature variable	
	int8_t allTimeMaxTemperature =  0;//Alltime max temperature variable
	uint8_t deviceAddress[8] = { 0 };//Array that contains the DOW address			
	uint8_t maxTemperatureFlag = 0;//Flag to check if a new max temperature was achieved
	uint8_t previousdayletters = 0;//Amount of letters used to spell the current day
	uint8_t seconds = 0, minutes = 0, hours = 0;//Data taken for the clock
	uint32_t error = 0;//Error variable 	
	time_t temperatureTimer = 0, previousSeconds = 0;//Timer variables to display temperature readings	
	char day[10] = "          ";//Buffer used to hold the "day" string
	char previousDay[10] = "          ";//Buffer used to hold the "previousDay" string
	char date[9] = "         ";//Buffer used to hold the "date" string
	char amPm[3] = "   ";//Buffer used to hold the "amPm" string	
	
	//Check the port for the current module	
	error = CreatePort();
	if (error != 0) 
	{
		return 1; 
	}
	error = SetLCDPort();
	if (error != 0)
	{ 
		return 1; 
	}
	ReadDisplayModule();
	if (ClearMaxTemperature)
	{
		ClearCustomerData();//Clear the Customer data		
	}	
	InitializeMediumNumbers();//Initialize medium numbers
	fprintf(stderr, "Press any key and enter to quit\n");
	WriteString("Running Demo...\n");		
	SearchForDowAddress(deviceAddress);//Search for DOW devices		
	currentTemperature = ReadTemperatureProbe(deviceAddress);//Interact with DOW device
	Sleep(500);//Allow time for the DOW device to initialize, avoiding incorrect data return
	while (!_kbhit())//Run until a key on the keyboard is pressed
	{		
		//Read the temperature probe for the current temperature
		currentTemperature = ReadTemperatureProbe(deviceAddress);//Read the temperature from the temperature probe		
		//Check if a new all time max temperature has been read				
		maxTemperatureFlag = CheckMaxTemperature(&currentTemperature, &allTimeMaxTemperature);		
		//Display the proper temperature alert if a new max temperature, or a new All-time max temperature has been measured.
		if (maxTemperatureFlag == 1)
		{			
			SaveAllTimeMaxTemperature(&allTimeMaxTemperature);
			DisplayAllTimeMaxTemperature(allTimeMaxTemperature);			
		}		
		//Get time and date		
		temperatureTimer = GetTime(&seconds, &minutes, &hours, amPm);	
		//display the temperature reading every 20 seconds
		if (temperatureTimer - previousSeconds >= 20)
		{
			previousSeconds = temperatureTimer;
			//Display the current temperature and the max temperature recorded during this session	
			DisplayTemperatureReading(currentTemperature, allTimeMaxTemperature);
		}
		GetDate(day, date);
		//Display the current time and date
		DisplayTime(seconds,minutes,hours,amPm);
		DisplayDate(day,date,previousDay);	
	}
	return 0;
}
int CreatePort(void)
{
	uint32_t error = 0;
	//Create an interface on the specified port
	error = CreateInterface(PORTNAME);	
	//alert user if an error occured during interface creation
	if (error != 0)
	{
		fprintf(stderr, "Error opening port!\nPlease connect your display, or change the COM port specified\nThen restart the program\nPress any key to close the program");		
		char c = getchar();
		while (c != '\n')
		{
			c = getchar();
		}
		return 1;
	}
	return 0;
}
int SetLCDPort(void)
{
	uint32_t error = 0;
	char c;
		
	error = SetInterfaceParameters(baud);
	//Alert user if the baud rate cannot be set properly
	if (error != 0)
	{
		fprintf(stderr, "Error setting port parameters!\nPress any Key to close the program");
		c = getchar();
		while (c != '\n')
		{
			c = getchar();
		}
		return 1;
	}
	error = SetInterfaceTimeouts(1000);
	//Alert user that a timeout has occured
	if (error != 0)
	{
		fprintf(stderr, "Error setting port timeouts!\nPress any Key to close the program");
		c = getchar();
		while (c != '\n')
		{
			c = getchar();
		}
		return 1;
	}
	return 0;
}
void ReadDisplayModule(void)
{
	uint8_t Module[] = { 0 };

    WriteString("Reading...");
    ReadModuleType(Module, 1);//Read which module has been connected		
	ClearScreen();
	//Specify which display has been connected
    switch (Module[0])
    {
    case 46:
        SetCursorPosition(1, 1);
        fprintf(stderr, "OK202-25-USB is connected\n");        
        break;
	//Alert the user that the module was unidentafiable
    default:
        fprintf(stderr,"Unknown Module Please connect and try again.\n");
		break;
    }
}
void ClearCustomerData(void)
{
	uint8_t customerData[16] = { 0 };//Make all values in the array equal to 0.
	WriteCustomerData(customerData, 16);//Write all zeros to Customer Data
}
void SearchForDowAddress(uint8_t deviceAddress[])
{
    uint8_t searchData[14] = { 0 };//Data recieved after searching for a DOW device	

    //Searches for a one wire command. Returned bits are stored in the searchData 14 bytes are returned
    SearchforaOneWireDevice(searchData, 14);		
    if (searchData[4] != 0)//Check if the search was successful
    {
        fprintf(stderr, "No Dallas One-Wire devices found!\n");//Alerts user that there is an issue
        fprintf(stderr, "Error code %d\n", searchData[4]);//Provides error code 
    }
    else//If there is no error code, the program will proceed
    {
        if (searchData[2] == 138)//Check if there are additional packets
        {
            fprintf(stderr, "An additional device packet was found!");
        }
        fprintf(stderr, "Packet type is %d\n", searchData[3]);//Alerts user of the packet type
        for (int i = 0; i < 8; i++)//Store the 8 byte device address
        {
            deviceAddress[i] = searchData[i+5];
        }
    }
}
int8_t ReadTemperatureProbe(uint8_t deviceAddress[])
{
	int8_t temperatureResponse[16] = { 0 };//Array storing data taken from the DOW device
    uint8_t initialSendBits[2] = {204, 68};//204 is the Skip ROM byte, while 68 is the convert temperature byte
    uint8_t convertResponse[5] = {0};//Used to store the 5 bytes returned (preamble, device packet, error code and CRC8 bytes)
	int8_t temperature = 0;//temperature value returned from the DOW temperature probe
	//Temp command is an array that contains the Match Rom command(85), along with the 8 bit device address, and the Read rom command (190).
    uint8_t tempCommand[10] = {85, 0, 0, 0, 0, 0, 0, 0, 0, 190};	

    DallasOneWireTransaction(initialSendBits, 2, 0, 16, 1);//Send data, length of data, receive bits, send bits, flags
    Read(convertResponse, 5);//Store the 5 bytes received into the convert_response array
    for (int i = 0; i < 8; i++)
    {
        tempCommand[i+1] = deviceAddress[i];		
    }
    DallasOneWireTransaction(tempCommand, 10, 72, 80, 3);//Send data, length of 10 bytes, receive 72 bits, send 80 bits, flag 3
	//The data bytes read are a Signature byte, Signature byte, Length byte, Type byte, Status byte, 9 data bytes, CRC Status byte
	Read(temperatureResponse, 15);  
	temperature = temperatureResponse[5] >> 1;
	if (temperatureResponse[6] == 0)//Extract temperature value from temperatureResponse array
	{
		temperature = temperatureResponse[5] >> 1;
	}
	else
	{
		temperature = temperatureResponse[5] >> 1;
		temperature *= -1;
	}
	return temperature;//return temperature value 
}
void DisplayTemperatureReading(int8_t currentTemperature, int8_t allTimeMaxTemperature)
{
	char currentTemperatureBuffer[20];
	char allTimeMaxTemperatureBuffer[20];

	ClearScreen();
	//display current temperature message	
	sprintf(currentTemperatureBuffer, "Temperature is %.2dC", currentTemperature);//Convert the data to a string
	SetCursorPosition(1, 1);//Set cursor in proper position
	WriteString(currentTemperatureBuffer);//Write the string on screen	
	//display allTimeMax temperature 
	sprintf(allTimeMaxTemperatureBuffer, "All time Max = %.2dC", allTimeMaxTemperature);//Convert the data to a string 
	SetCursorPosition(1, 2);//Set cursor in proper position
	WriteString(allTimeMaxTemperatureBuffer);//Write the string on screen
	Sleep(2000);
	ClearScreen();
}
int8_t CheckMaxTemperature(int8_t *currentTemperature, int8_t *allTimeMaxTemperature)
{	
	int8_t storedAllTimeMaxTemperature[16] = { 0 };
	//Read the All time max temperature value from display
	ReadCustomerData(storedAllTimeMaxTemperature, 16);
	//Extract allTimeMax Temperature
	*allTimeMaxTemperature = storedAllTimeMaxTemperature[0];
	//Compare temperature's if the current temperature and max temperatures are positive
	if (*currentTemperature > *allTimeMaxTemperature)
	{
		*allTimeMaxTemperature = *currentTemperature;
		return 1;
	}		
	else
	{
		return 0;
	}
}
void SaveAllTimeMaxTemperature(int8_t *allTimeMaxTemperature)
{
	int8_t storedAllTimeMaxTemperature[16] = { 0 };
	storedAllTimeMaxTemperature[0] = *allTimeMaxTemperature;
	WriteCustomerData(storedAllTimeMaxTemperature, 16);
}
void DisplayAllTimeMaxTemperature(int8_t allTimeMaxTemperature)
{
	char allTimeAlertBuffer[20] = { "New Alltime max temp" };
	char allTimeMaxValueBuffer[20];

	ClearScreen();
	SetCursorPosition(1, 1);//Set cursor in proper position.
	WriteString(allTimeAlertBuffer);//Write the string on screen.
	SetCursorPosition(1, 2);//Set cursor in proper position.		
	sprintf(allTimeMaxValueBuffer, "All time max= %.2dC", allTimeMaxTemperature);//Convert the data to a string.
	WriteString(allTimeMaxValueBuffer);//Write the string on screen.	
	Sleep(2000);
	ClearScreen();
}
time_t GetTime(uint8_t *seconds, uint8_t *minutes, uint8_t *hours, char *amPm)
{
	time_t rawtime;
	struct tm *info;
	time(&rawtime);
	info = localtime(&rawtime);
	//Retrieve current time (seconds, minutes, hours)
	*seconds = info->tm_sec;
	*minutes = info->tm_min;
	*hours = info->tm_hour;
	strftime(amPm, 3, "%p", info);//Converts the AmPM value to a string.	
	//return the rawtime seconds value to determine when to display current temperature values.
	rawtime = time(NULL);
	return rawtime;
}
void GetDate(char day[], char date[])
{
	time_t rawtime;
	struct tm *info;
	time( &rawtime );
	info = localtime( &rawtime );
	strftime(day, 10, "%A", info);//Converts the day value to a string	
	strftime(date, 9, "%x", info);//Converts the Date value to a string		
}
void DisplayTime(uint8_t seconds, uint8_t minutes, uint8_t hours, char amPm[])
{
	uint8_t tensOfSeconds = 0, onesOfSeconds = 0; 
	uint8_t tensOfMinutes = 0, onesOfMinutes = 0; 
	uint8_t tensOfHours = 0, onesOfHours = 0;	

	tensOfSeconds = seconds / 10;//Stens is the 10s position 
	onesOfSeconds = seconds % 10;//Sones is the 1s position
	tensOfMinutes = minutes / 10;//Mtens is the 10s position
	onesOfMinutes = minutes % 10;//Mones is the 1s position
	tensOfHours = hours / 10;//Htens is the 10s position
	onesOfHours = hours % 10;//Hones is the 1s position	
	PlaceMediumNumbers(1, 1, tensOfHours);//Places Htens digit in column 1, row 1
	PlaceMediumNumbers(1, 2, onesOfHours);//Places Hones digit in column 2, row 1
	PlaceMediumNumbers(1, 4, tensOfMinutes);//Places Mtens digit in column 4, row 1
	PlaceMediumNumbers(1, 5, onesOfMinutes);//Places Mones digit in column 5, row 1
	PlaceMediumNumbers(1, 7, tensOfSeconds);//Places Stens digit in column 7, row 1
	PlaceMediumNumbers(1, 8, onesOfSeconds);//Places Sones digit in column 8, row 1	
	SetCursorPosition(9, 1);//Sets the cursor position for the AmPm variable
	WriteString(amPm);//Writes the AmPm variable beside the time.
	DrawColon();
}
void EraseDay(char day[], char previousDay[])
{
	if (strlen(previousDay) != strlen(day))//if time has transitioned to a new day erase the last day
	{
		SetCursorPosition(11, 1);
		WriteString("          ");//erase day
		strcpy(previousDay, day);
	}
}
void DisplayDate(char day[], char date[], char previousDay[])
{
	uint8_t characterPlacement;		

	characterPlacement = 20 - strlen(day) + 1;//calculate where to place the first letter of the day	
	SetCursorPosition(characterPlacement, 1);//Sets the cursor in the proper position
	WriteString(day);//Writes the day		
	SetCursorPosition(13, 2);
	WriteString(date);//Writes the Date in mm/dd/yy format
}
void DrawColon(void)
{
	SetCursorPosition(3, 1);
	WriteString(".");//Draws the top halves of a colon in rows 3 and 5
	SetCursorPosition(6, 1);
	WriteString(".");//Draws the top halves of a colon in rows 3 and 5
	SetCursorPosition(3, 2);
	WriteString(".");//Draws the bottom halves of a colon in rows 3 and 5
	SetCursorPosition(6, 2);
	WriteString(".");//Draws the top halves of a colon in rows 3 and 5
}